import { groupBy } from './group-by';
import { compose } from './compose';
function createKeyFinder(map) {
    return function (key) {
        var result = map[key];
        if (!result) {
            throw new Error("Unexpected dictionary key encountered: '" + key + "'");
        }
        return result;
    };
}
function createDefinitionFinder(definitions) {
    var bySerializedKey = groupBy(definitions, function (t) { return t.serializedKey; });
    var byParsedKey = groupBy(definitions, function (t) { return t.parsedKey; });
    return {
        serializer: createKeyFinder(byParsedKey),
        parser: createKeyFinder(bySerializedKey)
    };
}
export function createDictionaryHandler(definitions) {
    var find = createDefinitionFinder(definitions);
    var useParsedKey = compose(function (d) { return d.parsedKey; }, find.parser);
    var useSerializedKey = compose(function (d) { return d.serializedKey; }, find.serializer);
    return {
        parse: function (input) {
            var parseItem = compose(function (d) { return d.parse(input[d.serializedKey]); }, find.parser);
            return groupBy(Object.getOwnPropertyNames(input), useParsedKey, parseItem);
        },
        serialize: function (input) {
            var serializeItem = compose(function (d) { return d.serialize(input[d.parsedKey]); }, find.serializer);
            return groupBy(Object.getOwnPropertyNames(input), useSerializedKey, serializeItem);
        }
    };
}
