"use strict";
var group_by_1 = require('./group-by');
var compose_1 = require('./compose');
function createKeyFinder(map) {
    return function (key) {
        var result = map[key];
        if (!result) {
            throw new Error("Unexpected dictionary key encountered: '" + key + "'");
        }
        return result;
    };
}
function createDefinitionFinder(definitions) {
    var bySerializedKey = group_by_1.groupBy(definitions, function (t) { return t.serializedKey; });
    var byParsedKey = group_by_1.groupBy(definitions, function (t) { return t.parsedKey; });
    return {
        serializer: createKeyFinder(byParsedKey),
        parser: createKeyFinder(bySerializedKey)
    };
}
function createDictionaryHandler(definitions) {
    var find = createDefinitionFinder(definitions);
    var useParsedKey = compose_1.compose(function (d) { return d.parsedKey; }, find.parser);
    var useSerializedKey = compose_1.compose(function (d) { return d.serializedKey; }, find.serializer);
    return {
        parse: function (input) {
            var parseItem = compose_1.compose(function (d) { return d.parse(input[d.serializedKey]); }, find.parser);
            return group_by_1.groupBy(Object.getOwnPropertyNames(input), useParsedKey, parseItem);
        },
        serialize: function (input) {
            var serializeItem = compose_1.compose(function (d) { return d.serialize(input[d.parsedKey]); }, find.serializer);
            return group_by_1.groupBy(Object.getOwnPropertyNames(input), useSerializedKey, serializeItem);
        }
    };
}
exports.createDictionaryHandler = createDictionaryHandler;
